@@ -86,7 +86,7 @@ package Astro::SpaceTrack;
 use base qw{Exporter};
 use vars qw{$VERSION @EXPORT_OK};
 
-$VERSION = "0.019";
+$VERSION = "0.020";
 @EXPORT_OK = qw{shell};
 
 use Astro::SpaceTrack::Parser;
@@ -170,7 +170,6 @@ my %catalogs = (	# Catalog names (and other info) for each source.
 	},
     );
 
-
 my %mutator = (	# Mutators for the various attributes.
     addendum => \&_mutate_attrib,		# Addendum to banner text.
     banner => \&_mutate_attrib,
@@ -281,10 +280,10 @@ return $self;
 }
 
 
-=for comment Help syntax-highlighting editor. "
-
 =item $resp = $st->amsat ()
 
+=for comment Help syntax-highlighting editor. "
+
 This method downloads current orbital elements from the Radio Amateur
 Satellite Corporation's web page, L<http://www.amsat.org/>. This lists
 satellites of interest to radio amateurs, and appears to be updated
@@ -331,16 +330,26 @@ $self->_dump_headers ($resp) if $self->{dump_headers};
 $resp;
 }
 
+=item @names = $st->attribute_names
+
+This method returns a list of legal attribute names.
+
+=cut
+
+sub attribute_names {sort keys %mutator}
+
 
 =item $resp = banner ();
 
+=for comment help for syntax-highlighting editor "
+
 This method is a convenience/nuisance: it simply returns a fake
 HTTP::Response with standard banner text. It's really just for the
 benefit of the shell method.
 
-=cut
+=for comment help for syntax-highlighting editor "
 
-# Help for syntax-highlighting editor that does not understand POD '
+=cut
 
 sub banner {
 my $self = shift;
@@ -391,8 +400,6 @@ You can specify the L</retrieve> options on this method as well.
 
 =cut
 
-# Help for syntax-highlighting editor that does not understand POD '
-
 {	# Local symbol block.
 
 my %valid_type = ('text/plain' => 1, 'text/text' => 1);
@@ -502,6 +509,8 @@ return $self->_handle_observing_list ($opt, <$fh>)
 
 =item $resp = $st->get (attrib)
 
+=for comment help syntax-highlighting editor "
+
 B<This method returns an HTTP::Response object> whose content is the value
 of the given attribute. If called in list context, the second element
 of the list is just the value of the attribute, for those who don't want
@@ -509,9 +518,9 @@ to winkle it out of the response object. We croak on a bad attribute name.
 
 See L</Attributes> for the names and functions of the attributes.
 
-=cut
+=for comment help syntax-highlighting editor "
 
-# Help for syntax-highlighting editor that does not understand POD '
+=cut
 
 sub get {
 my $self = shift;
@@ -530,13 +539,15 @@ return wantarray ? ($resp, $self->{$name}) : $resp;
 
 =item $resp = $st->help ()
 
+=for comment help syntax-highlighting editor "
+
 This method exists for the convenience of the shell () method. It
 always returns success, with the content being whatever it's
 convenient (to the author) to include.
 
-=cut
+=for comment help syntax-highlighting editor "
 
-# Help for syntax-highlighting editor that does not understand POD '
+=cut
 
 sub help {
 my $self = shift;
@@ -584,7 +595,7 @@ The following commands are defined:
       banner = false to supress the shell () banner;
       cookie_expires = Perl date the session cookie expires;
       direct = true to fetch orbital elements directly
-        from a redistributor. Currently this only affects the
+        from a redistributer. Currently this only affects the
         celestrak() method. The default is false.
       dump_headers is unsupported, and intended for debugging -
         don't be suprised at anything it does, and don't rely
@@ -623,6 +634,8 @@ eod
 
 =item $resp = $st->iridium_status ();
 
+=for comment help syntax-highlighting editor "
+
 This method queries Mike McCants' "Status of Iridium Payloads" web
 page, http://users2.ev1.net/~mmccants/tles/iridium.html (which gives
 status on non-function Iridium satellites) and the Celestrak list of
@@ -641,14 +654,14 @@ follows:
  '?' - not at operational altitude
  'man' - maneuvering, at least slightly.
 
-In addition, the data from Celestrak may contain the fillowing
+In addition, the data from Celestrak may contain the following
 status:
 
  'dum' - Dummy mass
 
-=cut
+=for comment help syntax-highlighting editor "
 
-# Help editor that does not understand POD. '
+=cut
 
 sub iridium_status {
 my $self = shift;
@@ -685,6 +698,8 @@ $resp;
 
 =item $resp = $st->login ( ... )
 
+=for comment help syntax-highlighting editor "
+
 If any arguments are given, this method passes them to the set ()
 method. Then it executes a login. The return is normally the
 HTTP::Response object from the login. But if no session cookie was
@@ -694,6 +709,8 @@ a login is attempted without the username and password being set, the
 return is an HTTP::Response with an appropriate message and the
 code set to RC_PRECONDITION_FAILED from HTTP::Status (a.k.a. 412).
 
+=for comment help syntax-highlighting editor "
+
 =cut
 
 sub login {
@@ -732,12 +749,16 @@ HTTP::Response->new (RC_OK, undef, undef, "Login successful.\n");
 
 =item $resp = $st->names (source)
 
+=for comment help syntax-highlighting editor "
+
 This method retrieves the names of the catalogs for the given source,
 either 'celestrak' or 'spacetrack', in the content of the given
 HTTP::Response object. In list context, you also get a reference to
 a list of two-element lists; each inner list contains the description
 and the catalog name (suitable for inserting into a Tk Optionmenu).
 
+=for comment help syntax-highlighting editor "
+
 =cut
 
 sub names {
@@ -830,25 +851,7 @@ my $self = shift;
 delete $self->{_content_type};
 
 @_ = _parse_retrieve_args (@_) unless ref $_[0] eq 'HASH';
-my $opt = shift;
-
-foreach my $key (qw{end_epoch start_epoch}) {
-    next unless $opt->{$key};
-    next if ref $opt->{$key};
-    $opt->{$key} !~ m/\D/ or
-	$opt->{$key} =~ m/^(\d+)\D+(\d+)\D+(\d+)$/ and
-	    $opt->{$key} = eval {timegm (0, 0, 0, $3, $2-1, $1)} or
-	croak <<eod;
-Error - Illegal date '$opt->{$key}'. Valid dates are a number
-	(interpreted as a Perl date) or numeric year-month-day.
-eod
-    my ($opp, $off) = $key eq 'start_epoch' ?
-	(end_epoch => 1) : (start_epoch => -1);
-    unless ($opt->{$opp}) {
-	$opt->{$opp} = [gmtime (86400 * $off + $opt->{$key})];
-	}
-    $opt->{$key} = [gmtime  ($opt->{$key})];
-    }
+my $opt = _parse_retrieve_dates (shift);
 
 $opt->{sort} ||= 'catnum';
 
@@ -929,6 +932,8 @@ $resp;
 
 =item $resp = $st->search_id (id ...)
 
+=for comment help syntax-highlighting editor "
+
 This method searches the database for objects having the given
 international IDs. The international ID is the last two digits
 of the launch year (in the range 1957 through 2056), the
@@ -957,6 +962,8 @@ added to the HTTP::Response object returned.
 
 You can specify the L</retrieve> options on this method as well.
 
+=for comment help syntax-highlighting editor "
+
 =cut
 
 sub search_id {
@@ -1007,6 +1014,8 @@ wantarray ? ($resp, \@table) : $resp;
 
 =item $resp = $st->search_name (name ...)
 
+=for comment help syntax-highlighting editor "
+
 This method searches the database for the named objects. Matches
 are case-insensitive and all matches are returned. There is no
 mechanism to restrict the search to a given date range, on-orbit
@@ -1029,6 +1038,8 @@ added to the HTTP::Response object returned.
 
 You can specify the L</retrieve> options on this method as well.
 
+=for comment help syntax-highlighting editor "
+
 =cut
 
 sub search_name {
@@ -1077,6 +1088,8 @@ wantarray ? ($resp, \@table) : $resp;
 
 =item $st->set ( ... )
 
+=for comment help syntax-highlighting editor "
+
 This is the mutator method for the object. It can be called explicitly,
 but other methods as noted may call it implicitly also. It croaks if
 you give it an odd number of arguments, or if given an attribute that
@@ -1088,6 +1101,8 @@ error we croak.
 
 See L</Attributes> for the names and functions of the attributes.
 
+=for comment help syntax-highlighting editor "
+
 =cut
 
 sub set {
@@ -1109,6 +1124,8 @@ HTTP::Response->new (RC_OK, undef, undef, COPACETIC);
 
 =item $st->shell ()
 
+=for comment help syntax-highlighting editor "
+
 This method implements a simple shell. Any public method name except
 'new' or 'shell' is a command, and its arguments if any are parameters.
 We use Text::ParseWords to parse the line, and blank lines or lines
@@ -1140,9 +1157,9 @@ one of the arguments was 'exit'.
 
 Unlike most of the other methods, this one returns nothing.
 
-=cut
+=for comment help syntax-highlighting editor "
 
-# Help for syntax-highlighting editor that does not understand POD '
+=cut
 
 my ($read, $print, $out, $rdln);
 
@@ -1158,7 +1175,8 @@ my $prompt = 'SpaceTrack> ';
 $out = \*STDOUT;
 $print = sub {
 	my $hndl = UNIVERSAL::isa ($_[0], 'FileHandle') ? shift : $out;
-	print $hndl @_};
+	print $hndl @_
+	};
 
 unshift @_, 'banner' if $self->{banner} && !$self->{filter};
 while (1) {
@@ -1246,6 +1264,8 @@ $self->shell ($self->_source (@_), 'exit');
 
 =item $resp = $st->spaceflight ()
 
+=for comment help syntax-highlighting editor "
+
 This method downloads current orbital elements from NASA's human
 spaceflight site, L<http://spaceflight.nasa.gov/>. As of November 2005
 you get the International Space Station. An attempt is made to get the
@@ -1261,24 +1281,37 @@ This method is a web page scraper. any change in the location of the
 web pages, or any substantial change in their format, will break this
 method.
 
+You can specify the L</retrieve> options on this method as well.
+
+=for comment help syntax-highlighting editor "
 
 =cut
 
 
-# Help editor that does not understand POD '
-
 sub spaceflight {
 my $self = shift;
 delete $self->{_content_type};
+
+@_ = _parse_retrieve_args (@_) unless ref $_[0] eq 'HASH';
+my $opt = _parse_retrieve_dates (shift, {perldate => 1});
+
+$opt->{sort} ||= 'catnum';
+
+$opt->{sort} eq 'catnum' || $opt->{sort} eq 'epoch' or die <<eod;
+Error - Illegal sort '$opt->{sort}'. You must specify 'catnum'
+        (the default) or 'epoch'.
+eod
+
 my $content = '';
 my $now = time ();
+my %tle;
 foreach my $url (
 	'http://spaceflight.nasa.gov/realdata/sightings/SSapplications/Post/JavaSSOP/orbit/ISS/SVPOST.html',
 	'http://spaceflight.nasa.gov/realdata/sightings/SSapplications/Post/JavaSSOP/orbit/SHUTTLE/SVPOST.html',
 	) {
     my $resp = $self->{agent}->get ($url);
     return $resp unless $resp->is_success;
-    my ($tle, @data, $epoch, $acquire);
+    my (@data, $acquire);
     foreach (split '\n', $resp->content) {
 	chomp;
 	m/TWO LINE MEAN ELEMENT SET/ and do {
@@ -1294,20 +1327,35 @@ foreach my $url (
 	    @data == 2 || @data == 3 or next;
 	    shift @data
 		if @data == 3 && !$self->{direct} && !$self->{with_name};
-	    my $yr = substr ($data[@data - 2], 18, 2);
-	    my $da = substr ($data[@data - 2], 20, 12);
+	    my $ix = @data - 2;
+	    my $id = substr ($data[$ix], 2, 5) + 0;
+	    my $yr = substr ($data[$ix], 18, 2);
+	    my $da = substr ($data[$ix], 20, 12);
 	    $yr += 100 if $yr < 57;
 	    my $ep = timegm (0, 0, 0, 1, 0, $yr) + ($da - 1) * 86400;
-	    next if $ep > $now;
-	    next if defined $epoch && $ep < $epoch;
-	    $tle = join '', @data;
+	    unless ($opt->{start_epoch} ?
+		    $ep > $opt->{end_epoch} || $ep <= $opt->{start_epoch} :
+		    $ep > $now) {
+		$tle{$id} ||= [];
+		my @keys = $opt->{descending} ? (-$id, -$ep) : ($id, $ep);
+		@keys = reverse @keys if $opt->{sort} eq 'epoch';
+		push @{$tle{$id}}, [@keys, join '', @data];
+		}
 	    @data = ();
-	    $epoch = $ep;
 	    };
 	}
-    $content .= $tle if $tle;
     }
 
+unless ($opt->{start_epoch}) {
+    my $left = $opt->{last5} ? 5 : 1;
+    foreach (values %tle) {splice @$_, $left}
+    }
+$content .= join '',
+    map {$_->[2]}
+    sort {$a->[0] <=> $b->[0] || $a->[1] <=> $b->[1]}
+    map {@$_} values %tle;
+
+
 $content or
     return HTTP::Response->new (RC_PRECONDITION_FAILED, NO_CAT_ID);
 
@@ -1415,8 +1463,7 @@ return $expir || 0;
 my $lookfor = $^O eq 'MacOS' ? qr{\012|\015+} : qr{\r\n};
 sub _convert_content {
 my $self = shift;
-local $/;
-$/ = undef;	# Slurp mode.
+local $/ = undef;	# Slurp mode.
 foreach my $resp (@_) {
     my $buffer = $resp->content;
     $buffer =~ s|$lookfor|\n|gms;
@@ -1642,6 +1689,53 @@ eod
 @_;
 }
 
+#	$opt = _parse_retrieve_dates ($opt);
+
+#	This subroutine looks for keys start_epoch and end_epoch in the
+#	given option hash, parses them as YYYY-MM-DD (where the letters
+#	are digits and the dashes are any non-digit punctuation), and
+#	replaces those keys' values with a reference to a list
+#	containing the output of timegm() for the given time. If only
+#	one epoch is provided, the other is defaulted to provide a
+#	one-day date range. If the syntax is invalid, we croak.
+#
+#	The return is the same hash reference that was passed in.
+
+sub _parse_retrieve_dates {
+my $opt = shift;
+my $ctl = shift || {};
+
+my $found;
+foreach my $key (qw{end_epoch start_epoch}) {
+    next unless $opt->{$key};
+    $opt->{$key} !~ m/\D/ or
+	$opt->{$key} =~ m/^(\d+)\D+(\d+)\D+(\d+)$/ and
+	    $opt->{$key} = eval {timegm (0, 0, 0, $3, $2-1, $1)} or
+	croak <<eod;
+Error - Illegal date '$opt->{$key}'. Valid dates are a number
+	(interpreted as a Perl date) or numeric year-month-day.
+eod
+    $found++;
+    }
+
+if ($found) {
+    if ($found == 1) {
+	$opt->{start_epoch} ||= $opt->{end_epoch} - 86400;
+	$opt->{end_epoch} ||= $opt->{start_epoch} + 86400;
+	}
+    $opt->{start_epoch} <= $opt->{end_epoch} or croak <<eod;
+Error - End epoch must not be before start epoch.
+eod
+    unless ($ctl->{perldate}) {
+	foreach my $key (qw{start_epoch end_epoch}) {
+	    $opt->{$key} = [gmtime ($opt->{$key})];
+	    }
+	}
+    }
+
+$opt;
+}
+
 #	_post is just like _get, except for the method used. DO NOT use
 #	this method in the login () method, or you get a bottomless
 #	recursion.
@@ -1723,7 +1817,7 @@ matches the cookie.
 =item direct (boolean)
 
 This attribute specifies that orbital elements should be fetched
-directly from the redistributor if possible. At the moment the only
+directly from the redistributer if possible. At the moment the only
 methods affected by this are celestrak() and spaceflight().
 
 The default is false (i.e. 0).
@@ -1783,7 +1877,7 @@ Anyone else will probably need to name an actual browser.
 This attribute specifies whether the returned element sets should
 include the common name of the body (three-line format) or not
 (two-line format). It is ignored if the 'direct' attribute is true;
-in this case you get whatever the redistributor provides.
+in this case you get whatever the redistributer provides.
 
 The default is false (i.e. 0).
 
@@ -1908,6 +2002,10 @@ insufficiently-up-to-date version of LWP or HTML::Parser.
    Added amsat() method.
  0.019 11-Jun-2006 T. R. Wyant
    Added the retrieve() options to celestrak() and file().
+ 0.020 01-Jul-2006 T. R. Wyant
+   Add the retrieve() qualifiers to spaceflight().
+   Add the attribute_names() method.
+   Tweak docs, correct spelling.
 
 =head1 ACKNOWLEDGMENTS
 
